package org.freihoff.yawl.persistence.adapter;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Properties;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import org.freihoff.yawl.persistence.generated.EmployeeType;

/**
 * Copyright 2014 Christian Freihoff
 *
 * @author Christian Freihoff
 */
public class EmployeeAdapter {

    private EntityManagerFactory entityManagerFactory;
    
    public EmployeeAdapter() {
        setPersistenceProperties();
    }

    public Long persistEmployee(EmployeeType employee) {
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        entityManager.getTransaction().begin();
        entityManager.persist(employee);
        entityManager.getTransaction().commit();
        entityManager.close();
        return employee.getHjid();
    }

    public void updateEmployee(EmployeeType employee) {
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        entityManager.getTransaction().begin();
        entityManager.merge(employee);
        entityManager.getTransaction().commit();
        entityManager.close();
    }

    public EmployeeType getEmployee(Long hjid) {
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        EmployeeType employee = entityManager.find(EmployeeType.class, hjid);
        entityManager.close();
        return employee;
    }

    public List<EmployeeType> getAllEmployees() {
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        List<EmployeeType>list = entityManager.createQuery("select e from EmployeeType e").getResultList();
        entityManager.close();
        return list;
    }

    public List<EmployeeType> getEmployeesByName(String name) {
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        List <EmployeeType> list = entityManager.createQuery("select e from EmployeeType e where lower(e.lastName) like :name or lower(e.firstName) like :name")
                .setParameter("name", "%"+name+"%")
                .getResultList();
        entityManager.close();
        return list;
    }
    
    public void deleteEmployee(Long hjid){
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        EmployeeType employee = entityManager.find(EmployeeType.class, hjid);
        entityManager.getTransaction().begin();
        entityManager.remove(employee);
        entityManager.getTransaction().commit();
        entityManager.close();   
    }
    
    public void setBlocker(Long hjid, boolean value){
        final EntityManager entityManager = entityManagerFactory.createEntityManager();
        EmployeeType employee = entityManager.find(EmployeeType.class, hjid);
        employee.setBlocked(value);
        entityManager.getTransaction().begin();
        entityManager.merge(employee);
        entityManager.getTransaction().commit();
        entityManager.close(); 
    }
    
    public void close(){
        entityManagerFactory.close();
    }

    private void setPersistenceProperties() {
        final Properties persistenceProperties = new Properties();
        InputStream inputStream;
        inputStream = getClass().getClassLoader().getResourceAsStream("persistence.properties");
        try {
            persistenceProperties.load(inputStream);
        } catch (IOException ioex) {
            //todo
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException ignored) {
                }
            }
        }
        entityManagerFactory = Persistence.createEntityManagerFactory("org.freihoff.yawl.persistence.generated", persistenceProperties);
    }
}
